@@ -15,7 +15,7 @@ __PACKAGE__->result_source_instance->view_definition(<<'ENDSQL');
     d.ip, d.name, d.dns,
     p.port, p.name AS port_description,
     p.remote_ip, p.remote_id, p.remote_type, p.remote_port,
-    dpp.remote_is_wap, dpp.remote_is_phone,
+    dpp.remote_is_discoverable, dpp.remote_is_wap, dpp.remote_is_phone,
     l.log AS comment,
     a.log, a.finished
 
@@ -31,6 +31,7 @@ __PACKAGE__->result_source_instance->view_definition(<<'ENDSQL');
 
   WHERE
     ds.device IS NULL
+    AND dpp.remote_is_discoverable
     AND ((p.remote_ip NOT IN (SELECT alias FROM device_ip))
          OR ((p.remote_ip IS NULL) AND p.is_uplink))
 
@@ -60,6 +61,8 @@ __PACKAGE__->add_columns(
   { data_type => "text", is_nullable => 1 },
   "remote_id",
   { data_type => "text", is_nullable => 1 },
+  "remote_is_discoverable",
+  { data_type => "boolean", is_nullable => 1 },
   "remote_is_wap",
   { data_type => "boolean", is_nullable => 1 },
   "remote_is_phone",
