@@ -34,24 +34,39 @@ ajax '/ajax/data/device/:ip/snmptree/:base' => require_login sub {
     my $base = param('base');
     $base =~ m/^\.1(\.\d+)*$/ or send_error('Bad OID Base', 404);
 
-    my $items = _get_snmp_data($device->ip, $base);
-
     content_type 'application/json';
+
+    return to_json [{
+      text => 'No data for this device. You can request a snapshot in the Details tab.',
+      children => \0,
+      state => { disabled => \1 },
+      icon => 'icon-search',
+    }] unless schema('netdisco')->resultset('DeviceSnapshot')->find($device->ip);
+
+    return to_json [{
+      text => 'No MIB data. Please run `~/bin/netdisco-do loadmibs`.',
+      children => \0,
+      state => { disabled => \1 },
+      icon => 'icon-search',
+    }] unless schema('netdisco')->resultset('SNMPObject')->count();
+
+    my $items = _get_snmp_data($device->ip, $base);
     to_json $items;
 };
 
-# TODO add form option for limiting to this device, so leave :ip
-ajax '/ajax/data/device/:ip/typeahead' => require_login sub {
-    my $device = try { schema('netdisco')->resultset('Device')
-                                         ->find( param('ip') ) }
-       or send_error('Bad Device', 404);
-
+ajax '/ajax/data/snmp/typeahead' => require_login sub {
     my $term = param('term') or return to_json [];
-    $term = '%'. $term .'%';
 
-    my @found = schema('netdisco')->resultset('SNMPObject')
-      ->search({ leaf => { -ilike => $term } },
-               { rows => 25, columns => 'leaf' })
+    my $device = param('ip');
+    my $deviceonly = param('deviceonly');
+    my $table = ($deviceonly ? 'DeviceBrowser' : 'SNMPObject');
+
+    my @found = schema('netdisco')->resultset($table)
+      ->search({ -or => [ oid => $term,
+                          oid => { -like => ($term .'.%') },
+                          leaf => { -ilike => ('%'. $term .'%') } ],
+                 (($deviceonly and $device) ? (ip => $device) : ()), },
+               { rows => 25, columns => 'leaf', order_by => 'oid_parts' })
       ->get_column('leaf')->all;
     return to_json [] unless scalar @found;
 
@@ -59,25 +74,24 @@ ajax '/ajax/data/device/:ip/typeahead' => require_login sub {
     to_json [ sort @found ];
 };
 
-# TODO add form option for limiting to this device, so leave :ip
-ajax '/ajax/data/device/:ip/snmpnodesearch' => require_login sub {
-    my $device = try { schema('netdisco')->resultset('Device')
-                                         ->find( param('ip') ) }
-       or send_error('Bad Device', 404);
-
-    my $to_match = param('str');
+ajax '/ajax/data/snmp/nodesearch' => require_login sub {
+    my $to_match = param('str') or return to_json [];
     my $partial = param('partial');
-    my $excludeself = param('excludeself');
 
-    return to_json [] unless $to_match or length($to_match);
-    $to_match = $to_match . '%' if $partial;
     my $found = undef;
-
-    my $op = ($partial ? '-ilike' : '=');
-    $found = schema('netdisco')->resultset('SNMPObject')
-      ->search({ -or => [ oid => { $op => $to_match }, leaf => { $op => $to_match } ] },
-               { rows => 1, order_by => 'oid_parts' })->first;
-
+    if ($partial) {
+        $found = schema('netdisco')->resultset('SNMPObject')
+          ->search({ -or => [ oid => $to_match,
+                              oid => { -like => ($to_match .'.%') },
+                              leaf => { -ilike => ($to_match .'%') } ] },
+                   { rows => 1, order_by => 'oid_parts' })->first;
+    }
+    else {
+        $found = schema('netdisco')->resultset('SNMPObject')
+          ->search({ -or => [ oid => $to_match,
+                              leaf => $to_match ] },
+                   { rows => 1, order_by => 'oid_parts' })->first;
+    }
     return to_json [] unless $found;
 
     $found = $found->oid;
@@ -123,13 +137,6 @@ sub _get_snmp_data {
     my ($ip, $base, $recurse) = @_;
     my @parts = grep {length} split m/\./, $base;
 
-    return [{
-      text => 'No data for this device. You can request a snapshot in the Details tab.',
-      children => \0,
-      state => { disabled => \1 },
-      icon => 'icon-search',
-    }] unless schema('netdisco')->resultset('DeviceSnapshot')->find($ip);
-
     my %meta = map { ('.'. join '.', @{$_->{oid_parts}}) => $_ }
                schema('netdisco')->resultset('Virtual::FilteredSNMPObject')
                                  ->search({}, { bind => [
@@ -149,7 +156,9 @@ sub _get_snmp_data {
         (scalar @{$meta{$_}->{index}}
           ? (icon => 'icon-th'.($meta{$_}->{browser} ? ' text-info' : ' muted')) : ()),
 
-        (($meta{$_}->{num_children} == 0 and ($meta{$_}->{access} =~ m/^(?:read|write)/ or $meta{$_}->{oid_parts}->[-1] == 0))
+        (($meta{$_}->{num_children} == 0 and ($meta{$_}->{type}
+                                              or $meta{$_}->{access} =~ m/^(?:read|write)/
+                                              or $meta{$_}->{oid_parts}->[-1] == 0))
           ? (icon => 'icon-leaf'.($meta{$_}->{browser} ? ' text-info' : ' muted')) : ()),
 
         # jstree will async call to expand these, and while it's possible
